AppTitle = "TOM TREASURE v0.4" ; (c) 2000 by Philipp Lenssen www.outer-court.com
Include "header.bb"
Include "sprite.bb"
Include "cache.bb"
Include "show_text.bb"
Include "data.bb"

Global gameEvent
Global gameNewEvent
Dim levelMap.typeMap(TILES_X, TILES_Y)
Dim tempMap(TILES_X, TILES_Y)
Dim debug$(DEBUG_LINES)
Dim debugCounter(0)

main
End

Function main()
	initGraphics
	showCover
	While Not exitGame
		handleGame
		pressedEscape = Not showText(KEY_SPACE, "GAME OVER", "[SPACE] RESTARTS | [ESC] QUITS")
		exitGame = pressedEscape
	Wend
End Function

Function handleGame()
	game.typeGame = New typeGame
	prepareGame game
	While Not game\over
		handleLevel game
	Wend
	clearSprites sprite.typeSprite, False
	Delete game
End Function

Function handleLevel(game.typeGame)
	level.typeLevel = New typeLevel
	assignLevel game
	initLevel game\level, level
	initSprites game\level, sprite.typeSprite
	initMap game\level
	setEvent EVENT_LEVEL_STARTS
	handleLevelLoop sprite, backgroundImage, game, level
	Delete level
End Function

Function handleLevelLoop(sprite.typeSprite, backgroundImage, game.typeGame, level.typeLevel)
	shadeImage = getImageFromCache("file/shade.bmp")
	shadeImageSmall = getImageFromCache("file/shade_small.bmp")
	heartImage = getImageFromCache("file/misc/heart.bmp")
	timer = CreateTimer(TIMER_FREQUENCY)

	While Not (game\over Or game\newLevel <> NO_LEVEL Or game\restartLevel)
		startTime = MilliSecs
		handlePause game
		
		assignEvent
		handleSprites sprite
		handleGameInput game

		SetBuffer BackBuffer()
		drawMap game
		drawSprites sprite, shadeImage, shadeImageSmall
		showLifes game, heartImage
		handleDebug
		Flip
	
		handleMusic game
		handleFadeOutCounter game
		handleGameEvents game, level
		handleDebugCounter

		WaitTimer timer
	Wend
	
	FreeTimer timer
End Function

Function initGraphics()
	Graphics SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_DEPTH
	ClsColor 0, 0, 0
End Function

Function handleDebugCounter()
	debugCounter(0) = debugCounter(0) + 1
	If debugCounter(0) = 30000
		debugCounter(0) = 0
	EndIf
End Function

Function showCover()
	image = LoadImage("file\misc\cover.bmp")
	x = 200
	y = 440
	Color 0, 0, 0
	PlayMusic "file\sound\23_chambers.mid"

	For i = 1 To 2
		If i = 1
			SetBuffer BackBuffer()			
		Else ;If i = 2
			SetBuffer FrontBuffer()
		EndIf
		
		If i = 1 Or i = 2
			DrawBlock image, 0, 0
			Locate x + 190, y + LETTER_HEIGHT
			Print "CREATED BY PHILIPP LENSSEN"
		EndIf
		If i = 2
			Locate x + 90, y
			Print "HIT ANY KEY TO CONTINUE"
		EndIf
	Next

	showCover = True
	While showCover
		If GetKey()
			showCover = False
		ElseIf Not MusicPlaying()
			screenSaver = True
			showCover = False
		EndIf
		Flip
		Delay 500
	Wend
	FreeImage image
	
	If screenSaver
		pressedEscape = Not showText(KEY_SPACE, "www.outer-court.com", "[SPACE] STARTS GAME")
	EndIf
End Function

Function sleep()
	While Not GetKey()
	Wend
End Function

Function showLifes(game.typeGame, heartImage)
	heartWidth = 22
	For x = 0 To game\lifes - 1
		DrawImage heartImage, MENU_MARGIN + x * heartWidth, MENU_MARGIN
	Next
End Function

Function handleFadeOutCounter(game.typeGame)
	If game\fadeOutCounter > 0
		game\fadeOutCounter = game\fadeOutCounter - 1
		If game\fadeOutCounter = 0
			If game\lifes > 0
				game\newLevel = game\level
				game\level = NO_LEVEL
				game\restartLevel = True
			Else
				game\over = True
			EndIf
		EndIf
	EndIf
End Function

Function handleGameEvents(game.typeGame, level.typeLevel)
	Select gameEvent
		Case EVENT_LEVEL_STARTS
			pushPlayerInside
			Select game\level
				Case 1
					game\newBackgroundMusic$ = "file/sound/19_pirates.mid"
			End Select
		Case EVENT_PLAYER_DIES
			game\lifes = game\lifes - 1
			game\fadeOutCounter = 100
		Case EVENT_PLAYER_EXITS_TOP
			If level\exitTop = NO_LEVEL
				pushPlayerInside
			Else
				game\newLevel = level\exitTop
			EndIf
		Case EVENT_PLAYER_EXITS_RIGHT
			If level\exitRight = NO_LEVEL
				pushPlayerInside
			Else
				game\newLevel = level\exitRight
			EndIf
		Case EVENT_PLAYER_EXITS_BOTTOM
			If level\exitBottom = NO_LEVEL
				pushPlayerInside
			Else
				game\newLevel = level\exitBottom
			EndIf
		Case EVENT_PLAYER_EXITS_LEFT
			If level\exitLeft = NO_LEVEL
				pushPlayerInside
			Else
				game\newLevel = level\exitLeft
			EndIf
	End Select
End Function

Function assignLevel(game.typeGame)
	game\level = game\newLevel
	game\newLevel = NO_LEVEL
	game\restartLevel = False
End Function

Function initLevel(gameLevel, level.typeLevel)
	restoreLevel gameLevel
	Read level\exitTop, level\exitRight, level\exitBottom, level\exitLeft
End Function

Function handleGameInput(game.typeGame)
	If 	KeyDown(KEY_ESCAPE)
		game\over = True
	ElseIf KeyDown(KEY_TOGGLE_MUSIC)
		game\musicOn = Not game\musicOn
	ElseIf KeyDown(KEY_PAUSE)
		game\pause = True
	EndIf
End Function

Function handlePause(game.typeGame)
	If game\pause
		exitsWithEscape = Not showText(KEY_SPACE, "GAME PAUSED", "[SPACE] TO CONTINUE")
		game\pause = False
	EndIf
End Function

Function printCenter(printText$)
	; ;
	Locate SCREEN_WIDTH / 2, SCREEN_HEIGHT / 2
	Print printText$
End Function

Function handleMusic(game.typeGame)
	sound = NOTHING
	Select gameEvent
		Case EVENT_PLAYER_SHOOTS
			sound = getSoundFromCache("file\sound\bolt.wav", .85)
	End Select
	
	If game\musicOn 
		If Not sound = NOTHING
			PlaySound sound
		EndIf
	EndIf
	
	If game\newBackgroundMusic$ <> game\backgroundMusic$
		game\backgroundMusic$ = game\newBackgroundMusic$
		If game\musicOn
			PlayMusic game\backgroundMusic$
		EndIf
	EndIf
	If Not MusicPlaying()
		If game\musicOn
			PlayMusic game\backgroundMusic$
		EndIf
	EndIf
End Function

Function setEvent(event)
	If gameNewEvent = NO_EVENT
		gameNewEvent = event
	EndIf
End Function

Function assignEvent()
	gameEvent = gameNewEvent
	gameNewEvent = NO_EVENT
End Function

Function getEvent(event)
	Return (gameEvent = event)
End Function

Function prepareGame(game.typeGame)
	game\musicOn = True
	game\level = NO_LEVEL
	game\newLevel = 1
	game\backgroundMusic$ = ""
	game\newBackgroundMusic$ = ""
	game\restartLevel = False
	game\lifes = DEFAULT_LIFES
End Function

Function drawMap(game.typeGame)
	handleAnimationCounter game
	
	For tileX = 1 To TILES_X
		For tileY = 1 To TILES_Y
			x = (tileX - 1) * TILE_SIZE
			y = (tileY - 1) * TILE_SIZE
			If levelMap(tileX, tileY)\animation
				DrawBlock levelMap(tileX, tileY)\image, x, y, game\animation
			Else
				DrawBlock levelMap(tileX, tileY)\image, x, y
			EndIf
		Next
	Next
End Function

Function handleAnimationCounter(game.typeGame)
	game\animationCounter = game\animationCounter + 1
	If game\animationCounter = ANIMATION_DELAY
		game\animationCounter = 0
		If game\animation = 0
			game\animation = 1
		Else
			game\animation = 0
		EndIf
	EndIf
End Function

Function initMap(gameLevel)
	RestoreLevelTiles gameLevel
	readTempMap
	
	For tileX = 1 To TILES_X
		For tileY = 1 To TILES_Y
			levelMap.typeMap(tileX, tileY) = New typeMap
			restoreTile tempMap(tileX, tileY)
			
			Read imageName$, ground
			Read animation
			levelMap(tileX, tileY)\ground = ground
			levelMap(tileX, tileY)\animation = animation

			fileName$ = "file\tile\" + imageName$ + ".bmp"
			If animation
				levelMap(tileX, tileY)\image = getAnimImageFromCache(fileName$, TILE_SIZE, TILE_SIZE, 0, 2)
			Else
				levelMap(tileX, tileY)\image = getImageFromCache(fileName$)
			EndIf
		Next
	Next
End Function

Function initSprites(gameLevel, sprite.typeSprite)
	clearSprites sprite.typeSprite, True
	playerAlive = spriteClassExists(sprite, CLASS_PLAYER)
		
	RestoreLevelSprites gameLevel
	readTempMap
	
	For tileX = 1 To TILES_X
		For tileY = 1 To TILES_Y

			If Not tempMap(tileX, tileY) = 0
				restoreSprite tempMap(tileX, tileY)

				Read speedX#, speedY#, speedStep#, speedBrake#, speedLimit
				Read imageName$, width, height, class
				Read oneDirectionalPusher
				Read frameCells
				Read collideable, container, containable
				Read layer
				Read frameCounterMax

				If Not (class = CLASS_PLAYER And playerAlive)

					sprite.typeSprite = New typeSprite
					For sprite.typeSprite = Each typeSprite
						If Not sprite\born
							x = tileX * TILE_SIZE
							y = tileY * TILE_SIZE
							initSprite sprite.typeSprite, x, y, speedX#, speedY#, speedStep#, speedLimit, speedBrake#, imageName$, width, height, class, oneDirectionalPusher, frameCells, NOBODY, collideable, container, containable, layer, frameCounterMax
						EndIf
					Next

				EndIf
				
			EndIf
			
		Next
	Next
End Function

Function clearSprites(sprite.typeSprite, exceptPlayer)
	For sprite.typeSprite = Each typeSprite
		If (exceptPlayer And sprite\class = CLASS_PLAYER)
			spriteInvertPosition sprite
		Else
			Delete sprite
		EndIf
	Next
End Function

Function readTempMap()
	For tileY = 1 To TILES_Y
		For tileX = 1 To TILES_X
			Read tileNumber
			tempMap(tileX, tileY) = tileNumber
		Next
	Next
End Function

Function debugPrint(infoString$)
	freeIndex = -1
	For i = 1 To DEBUG_LINES
		If debug$(i) = ""
			freeIndex = i
		EndIf
	Next

	If freeIndex = -1
		For i = 1 To DEBUG_LINES-1
			debug$(i) = debug$(i+1)
		Next
		freeIndex = DEBUG_LINES
	EndIf

	debug$(freeIndex) = debugCounter(0) + ": " + infoString$
End Function

Function handleDebug()
	leftMargin = 10
	shadowOffset = 1
	For i = 1 To DEBUG_LINES
		Color 0, 0, 0
		Locate leftMargin + shadowOffset, i * LINE_HEIGHT + shadowOffset
		Print debug$(i)

		Color 255, 255, 255
		Locate leftMargin, i * LINE_HEIGHT
		Print debug$(i)
	Next
End Function

Function verboseClass$(classNumber)
	textOf$ = ""
	Select classNumber
		Case CLASS_PLAYER: textOf$ = "player"
		Case CLASS_GUARD: textOf$ = "guard"
		Case CLASS_BARRIER: textOf$ = "barrier"
		Case CLASS_BULLET: textOf$ = "bullet"
		Case CLASS_HUNTER: textOf$ = "hunter"
		Case CLASS_AIR: textOf$ = "air"
		Case CLASS_COWARD: textOf$ = "coward"
		Case CLASS_FOOL: textOf$ = "fool"
		Case CLASS_BALL: textOf$ = "ball"
		Case CLASS_BALL_BOUNCE: textOf$ = "ball bounce"
		Case CLASS_SLIDER_ONCE: textOf$ = "slider once"
		Case CLASS_SLIDER_RESTART: textOf$ = "slider restart"
		Case CLASS_SLIDER_BOUNCE: textOf$ = "slider bounce"
		Default: textOf$ = "unknown"
	End Select
	Return textOf$
End Function

Function setGround(sprite.typeSprite)
	If sprite\collideable
		If Not sprite\contained
			x = sprite\x# + sprite\width / 2
			y = sprite\y# + sprite\height
			tileX = Int(x / TILE_SIZE) + 1
			tileY = Int(y / TILE_SIZE) + 1
			If tileX >= 1 And tileX < TILES_X And tileY >= 1 And tileY <= TILES_Y
				sprite\ground = levelMap(tileX, tileY)\ground
			EndIf
		EndIf
	EndIf
End Function