Function handleSprites(sprite.typeSprite)
	spriteCollection.typeSprite = sprite
	For sprite.typeSprite = Each typeSprite
		If sprite\class = CLASS_PLAYER
			spritePlayer.typeSprite = sprite
		EndIf
	Next

	handleContainerAll sprite, spriteCollection

	For sprite.typeSprite = Each typeSprite
		handleClassMorph sprite
		handleGround sprite
		handleSpeed sprite
		handleCounter sprite
		spriteHandleCollision sprite, spriteCollection
		handleSpriteClassSpecific sprite, spriteCollection, spritePlayer
		keepSpeedLimit sprite
		breakSpeed sprite
		handleFrame sprite
	Next

	killDyingSprites sprite
End Function

Function handleSpriteClassSpecific(sprite.typeSprite, spriteCollection.typeSprite, spritePlayer.typeSprite)
	Select sprite\class
		Case CLASS_PLAYER
			spriteHandleDirectionInput sprite, PLAYER_KEY_LEFT, PLAYER_KEY_UP, PLAYER_KEY_RIGHT, PLAYER_KEY_DOWN, True
			spriteHandleActionInput sprite, PLAYER_KEY_ACTION, spriteCollection
			spriteHandlePushed sprite
			playerHandleExit sprite
			
		Case CLASS_DEAD
			If sprite\counter = COUNTER_END Then sprite\dying = True
			
		Case CLASS_FOOL
			If sprite\counter = COUNTER_END
				;debugPrint sprite\ground
				If Rnd(100) >= 70
					sprite\followTarget = True
					clearTargetSpeed sprite
					sprite\counter = Int(Rnd(50)) + 250
				Else
					sprite\followTarget = False
					If sprite\ground = GROUND_WATER
						reverseSpeedSoft sprite
						sprite\counter = Int(Rnd(50)) + 250
					Else		
						setRandomTargetSpeed sprite, 2
						sprite\counter = Int(Rnd(50)) + 50
					EndIf
				EndIf
			EndIf
			
			spriteBorderBounce sprite, True
			
		Case CLASS_BULLET
			dieOutside sprite
			
		Case CLASS_BALL, CLASS_BALL_BOUNCE
			spriteHandlePushedBall sprite
			handleBall sprite, (sprite\class = CLASS_BALL_BOUNCE)
			
		Case CLASS_GUARD
			If getEvent(EVENT_PLAYER_SHOOTS)
				morphTo sprite, CLASS_HUNTER, 200, NO_SPEED#, NO_SPEED#
			EndIf
			spriteBorderRotate sprite
			spriteHandlePushed sprite
			
		Case CLASS_SLIDER_BOUNCE
			spriteBorderBounce sprite, True
			handleBulletDefense sprite
			
		Case CLASS_SLIDER_RESTART
			spriteBorderRestart sprite
			
		Case CLASS_SLIDER_ONCE
			dieOutside sprite
			
		Case CLASS_HUNTER
			followTarget sprite, spritePlayer
			
	End Select
End Function

Function clearTargetSpeed(sprite.typeSprite)
	sprite\targetSpeedX = NO_SPEED#
	sprite\targetSpeedY = NO_SPEED#
End Function

Function followTarget(sprite.typeSprite, target.typeSprite)
	followTargetPosition sprite, target\x + target\width / 2, target\y + target\height / 2
End Function

Function reverseSpeedSoft(sprite.typeSprite)
	sprite\targetSpeedX# = sprite\speedX# * -1
	sprite\targetSpeedY# = sprite\speedY# * -1
End Function

Function setRandomTargetSpeed(sprite.typeSprite, amount)
	sprite\targetSpeedX# = Rnd(amount) * 2 - amount
	sprite\targetSpeedY# = Rnd(amount) * 2 - amount
End Function

Function setRandomSpeed(sprite.typeSprite, amount)
	sprite\speedX# = Rnd(amount) * 2 - amount
	sprite\speedY# = Rnd(amount) * 2 - amount
End Function

Function handleCounter(sprite.typeSprite)
	If sprite\counter >= COUNTER_END
		sprite\counter = sprite\counter - 1
	EndIf
End Function

Function dieOutside(sprite.typeSprite)
	If isOutside(sprite)
		sprite\dying = True
	EndIf
End Function

Function playerHandleExit(sprite.typeSprite)
	If sprite\x + sprite\width < SCREEN_LEFT
		If sprite\speedX < -SPEED_MIN#
			setEvent EVENT_PLAYER_EXITS_LEFT
		EndIf
	ElseIf sprite\x > SCREEN_RIGHT
		If sprite\speedX > SPEED_MIN#
			setEvent EVENT_PLAYER_EXITS_RIGHT
		EndIf
	ElseIf sprite\y + sprite\height < SCREEN_TOP
		If sprite\speedY < -SPEED_MIN#
			setEvent EVENT_PLAYER_EXITS_TOP
		EndIf
	ElseIf sprite\y > SCREEN_BOTTOM
		If sprite\speedY > SPEED_MIN#
			setEvent EVENT_PLAYER_EXITS_BOTTOM
		EndIf
	EndIf
End Function

Function handleContainerAll(sprite.typeSprite, spriteCollection.typeSprite)
	For sprite.typeSprite = Each typeSprite
		If sprite\contained Then sprite\contained = False
	Next
	For sprite.typeSprite = Each typeSprite
		handleContainer sprite, spriteCollection
	Next
End Function

Function handleContainer(sprite.typeSprite, spriteCollection.typeSprite)
	If sprite\container
		For spriteCollection.typeSprite = Each typeSprite
			If spriteCollection\containable
				If Not sprite = spriteCollection
				
					If spriteIsInside(sprite, spriteCollection)
						If sprite\speedX <> 0
							If spriteCollection\speedY <> 0
								spriteCollection\x = spriteCollection\x + sprite\speedX
							EndIf
						EndIf
						If sprite\speedY <> 0
							If spriteCollection\speedX <> 0
								spriteCollection\y = spriteCollection\y + sprite\speedY
							EndIf
						EndIf
						spriteCollection\contained = True
					EndIf
					
				EndIf
			EndIf
		Next
	EndIf
End Function

Function spriteIsInside(a.typeSprite, b.typeSprite)
	isInside = False
	bX = b\x# + b\width / 2
	If bX >= a\x And bX <= a\x + a\width
		bY = b\y# + b\height
		If bY >= a\y And bY <= a\y + a\height
			isInside = True
		EndIf
	EndIf
	Return isInside
End Function

Function handleBulletDefense(sprite.typeSprite)
	If sprite\pushedBy = CLASS_BULLET And sprite\pushedByMaster = CLASS_PLAYER
		morphTo sprite, CLASS_HUNTER, 200, NO_SPEED#, NO_SPEED#
	EndIf
End Function

Function morphTo(sprite.typeSprite, newClass, morphEnergy, newSpeedBrake#, newSpeedLimit#)
	sprite\oldClass = sprite\class
	sprite\classMorphEnergy = morphEnergy
	sprite\class = newClass

	If newSpeedBrake# <> NO_SPEED#
		sprite\oldSpeedBrake# = sprite\speedBrake#
		sprite\speedBrake# = newSpeedBrake#
	EndIf
	If newSpeedLimit# <> NO_SPEED#
		sprite\oldSpeedLimit# = sprite\speedLimit#
		sprite\speedLimit# = newSpeedLimit#
	EndIf
	
	If speedX# <> NO_SPEED
		sprite\speedX# = speedX#
	EndIf
	If speedY# <> NO_SPEED
		sprite\speedY# = speedY#
	EndIf
End Function

Function handleClassMorph(sprite.typeSprite)
	If sprite\classMorphEnergy > 0
		sprite\classMorphEnergy = sprite\classMorphEnergy - 1
		If sprite\classMorphEnergy = 0
			sprite\class = sprite\oldClass
			sprite\oldClass = NOBODY
			
			If sprite\oldSpeedLimit# <> NO_SPEED#
				sprite\speedLimit# = sprite\oldSpeedLimit#
			EndIf
			If sprite\oldSpeedBrake <> NO_SPEED#
				sprite\speedBrake# = sprite\oldSpeedBrake#
			EndIf
		EndIf
	EndIf
End Function

Function handleGround(sprite.typeSprite)
	If sprite\collideable
		setGround sprite
		If sprite\ground = GROUND_BARRIER
			spriteBump sprite
			spriteBounce sprite
		ElseIf sprite\ground = GROUND_DANGER Or sprite\ground = GROUND_WATER
			If Not (sprite\layer >= LAYER_NEAR_SKY And sprite\ground = GROUND_WATER)
				If sprite\energy > 0
					sprite\energy = sprite\energy - 1
					If sprite\energy = 0
						If sprite\class = CLASS_PLAYER
							setEvent EVENT_PLAYER_DIES
						EndIf
						transformToDead sprite
					EndIf
				EndIf
			EndIf
		EndIf
	EndIf
End Function

Function transformToDead(sprite.typeSprite)
	sprite\class = CLASS_DEAD
	sprite\counter = 50
	sprite\speedBrake# = .1
	sprite\speedY = -3
	sprite\speedX = 0
	sprite\speedLimit = 2
	sprite\animated = False
	sprite\image = getImageFromCache("file/skull.bmp")
	sprite\collideable = False
	sprite\containable = False
End Function

Function spriteBump(sprite.typeSprite)
	sprite\x = sprite\x + Sgn(sprite\speedX) * -(sprite\speedLimit * 2)
	sprite\y = sprite\y + Sgn(sprite\speedY) * -(sprite\speedLimit * 2)
End Function

Function spriteBounce(sprite.typeSprite)
	sprite\speedX = sprite\speedX * -1
	sprite\speedY = sprite\speedY * -1
End Function

Function killDyingSprites(sprite.typeSprite)
	For sprite.typeSprite = Each typeSprite	
		If sprite\dying Delete sprite
	Next	
End Function

Function handleBall(sprite.typeSprite, bounce)
	If Not sprite\pushedBy = NOBODY
		If sprite\pushedBy = CLASS_BARRIER
			If bounce
				sprite\speedX = sprite\speedX * -1
				sprite\speedY = sprite\speedY * -1
			Else
				sprite\speedX = 0
				sprite\speedY = 0
			EndIf
		Else
			If Abs(sprite\pushedX) > Abs(sprite\pushedY)
				sprite\speedX = sprite\pushedX * sprite\speedLimit
				sprite\speedY = 0
			Else
				sprite\speedX = 0
				sprite\speedY = sprite\pushedY * sprite\speedLimit
			EndIf
		EndIf
		sprite\pushedBy = NOBODY
	EndIf
End Function

Function spriteHandleCollision(sprite.typeSprite, spriteCollection.typeSprite)
	For spriteCollection.typeSprite = Each typeSprite
		If Not sprite = spriteCollection
			If sprite\collideable
				If spriteCollection\collideable
					checkCollision sprite, spriteCollection
				EndIf
			EndIf
		EndIf
	Next
End Function

Function breakSpeed(sprite.typeSprite)
	If sprite\speedX > SPEED_MIN#
		sprite\speedX = sprite\speedX - sprite\speedBrake
		If sprite\speedX <= SPEED_MIN# Then sprite\speedX = SPEED_MIN#
	ElseIf sprite\speedX < -SPEED_MIN#
		sprite\speedX = sprite\speedX + sprite\speedBrake
		If sprite\speedX >= -SPEED_MIN# Then sprite\speedX = -SPEED_MIN#
	EndIf
	If sprite\speedY > SPEED_MIN#
		sprite\speedY = sprite\speedY - sprite\speedBrake
		If sprite\speedY <= SPEED_MIN# Then sprite\speedY = SPEED_MIN#
	ElseIf sprite\speedY < -SPEED_MIN#
		sprite\speedY = sprite\speedY + sprite\speedBrake
		If sprite\speedY >= -SPEED_MIN# Then sprite\speedY = -SPEED_MIN#
	EndIf
End Function

Function spriteHandlePushedBall(sprite.typeSprite)
	If Not sprite\pushedBy = NOBODY
		If sprite\pushedX <> 0 Or sprite\pushedY <> 0
			If Abs(sprite\pushedX) > Abs(sprite\pushedY)
				If sprite\pushedBy = CLASS_BALL_BOUNCE
					sprite\speedX = sprite\pushedX
				Else
					sprite\speedX = Sgn(sprite\pushedX) * sprite\speedLimit
				EndIf
			Else
				If sprite\pushedBy = CLASS_BALL_BOUNCE
					sprite\speedY = sprite\pushedY
				Else
					sprite\speedY = Sgn(sprite\pushedY) * sprite\speedLimit
				EndIf
			EndIf
			sprite\pushedX = 0
			sprite\pushedY = 0
		EndIf
		sprite\pushedBy = NOBODY
	EndIf
End Function

Function spriteHandlePushed(sprite.typeSprite)
	If Not sprite\pushedBy = NOBODY
		If sprite\pushedX <> 0
			sprite\speedX = sprite\pushedX
			sprite\pushedX = 0
		EndIf
		If sprite\pushedY <> 0
			sprite\speedY = sprite\pushedY
			sprite\pushedY = 0
		EndIf
		sprite\pushedBy = NOBODY
	EndIf
End Function

Function checkCollision(a.typeSprite, b.typeSprite)
	If ImagesCollide(a\image, a\x, a\y, a\frame, b\image, b\x, b\y, b\frame)
		rightBorderIn = (a\x + a\width >= b\x And a\x + a\width <= b\x + b\width)
		If rightBorderIn
			leftBorderIn = False
		Else
			leftBorderIn = (a\x <= b\x + b\width And a\x >= b\x)
		EndIf
		bottomBorderIn = (a\y + a\height >= b\y And a\y + a\height <= b\y + b\height)
		If bottomBorderIn
			topBorderIn = False
		Else
			topBorderIn = (a\y <= b\y + b\height And a\y >= b\y)
		EndIf
		setPushed a, b, rightBorderIn, leftBorderIn, topBorderIn, bottomBorderIn
	EndIf
End Function

Function setPushed(a.typeSprite, b.typeSprite, rightBorderIn, leftBorderIn, topBorderIn, bottomBorderIn)
	If (rightBorderIn Or leftBorderIn) And (bottomBorderIn Or topBorderIn)
		If rightBorderIn
			a\pushedX = -1
			b\pushedX = 1
		ElseIf leftBorderIn
			a\pushedX = 1
			b\pushedX = -1
		EndIf
		If bottomBorderIn
			a\pushedY = -1
			b\pushedY = 1
		ElseIf topBorderIn
			a\pushedY = 1
			b\pushedY = -1
		EndIf
		handleOneDirectionalPusher a, b
		
		a\pushedBy = b\class
		a\pushedByMaster = b\master
		b\pushedBy = a\class
		b\pushedByMaster = a\master
	EndIf
End Function

Function handleOneDirectionalPusher(a.typeSprite, b.typeSprite)
	If b\oneDirectionalPusher
		If a\pushedX <> 0 And a\pushedY <> 0
			If Abs(b\speedX) > Abs(b\speedY)
				a\pushedY = 0
			Else
				a\pushedX = 0
			EndIf
		EndIf
	EndIf

	If a\oneDirectionalPusher
		If b\pushedX <> 0 And b\pushedY <> 0
			If Abs(a\speedX) > Abs(a\speedY)
				b\pushedY = 0
			Else
				b\pushedX = 0
			EndIf
		EndIf
	EndIf
End Function

Function spriteHandleActionInput(sprite.typeSprite, actionKey, spriteCollection.typeSprite)
	If sprite\pauseAction > 0
		sprite\pauseAction = sprite\pauseAction - 1
	Else
		If KeyDown(actionKey)
			bulletSprite.typeSprite = New typeSprite
			sprite\pauseAction = PLAYER_PAUSE_ACTION
			shootBullet sprite, bulletSprite
			setEvent EVENT_PLAYER_SHOOTS
		EndIf
	EndIf
End Function

Function shootBullet(sprite.typeSprite, bulletSprite.typeSprite)
	bulletSpeedX = Sgn(sprite\speedX) * 5
	If bulletSpeedX = 0
		bulletSpeedY = Sgn(sprite\speedY) * 5
	EndIf
	If bulletSpeedX = 0 And bulletSpeedY = 0
		bulletSpeedY = 5
	EndIf
	centerX = sprite\x + sprite\width / 2
	centerY = sprite\y + sprite\height / 2
	border = 4
	offX = Sgn(bulletSpeedX) * (sprite\width / 2 + border)
	offY = Sgn(bulletSpeedY) * (sprite\height / 2 + border)
	bulletSize = 16
	bulletX = centerX + offX - bulletSize / 2
	bulletY = centerY + offY - bulletSize / 2

	initSprite bulletSprite, bulletX, bulletY, bulletSpeedX, bulletSpeedY, .4, 10, 0, "object\orb", bulletSize, bulletSize, CLASS_BULLET, False, 0, CLASS_PLAYER, True, False, True, LAYER_NEAR_SKY, DEFAULT_FRAME_COUNTER_MAX
End Function

Function spriteHandleDirectionInput(sprite.typeSprite, leftKey, upKey, rightKey, downKey, oneDirectional)
	If KeyDown(leftKey)
		sprite\speedX = sprite\speedX - sprite\speedStep
		If oneDirectional sprite\speedY = 0
	ElseIf KeyDown(rightKey)
		sprite\speedX = sprite\speedX + sprite\speedStep
		If oneDirectional sprite\speedY = 0
	ElseIf KeyDown(upKey)
		sprite\speedY = sprite\speedY - sprite\speedStep
		If oneDirectional sprite\speedX = 0
	ElseIf KeyDown(downKey)
		sprite\speedY = sprite\speedY + sprite\speedStep
		If oneDirectional sprite\speedX = 0
	EndIf
End Function

Function keepSpeedLimit(sprite.typeSprite)
	If sprite\speedX < -sprite\speedLimit
		sprite\speedX = -sprite\speedLimit
		sprite\speedY = 0
	ElseIf sprite\speedX > sprite\speedLimit
		sprite\speedX = sprite\speedLimit
		sprite\speedY = 0
	EndIf
	If sprite\speedY < -sprite\speedLimit
		sprite\speedY = -sprite\speedLimit
		sprite\speedX = 0
	ElseIf sprite\speedY > sprite\speedLimit
		sprite\speedY = sprite\speedLimit
		sprite\speedX = 0
	EndIf
End Function

Function initSprite(sprite.typeSprite, x, y, speedX#, speedY#, speedStep#, speedLimit, speedBrake#, imageName$, width, height, spriteClass, oneDirectionalPusher, frameCells, master, collideable, container, containable, layer, frameCounterMax)
	sprite\frame = 0
	sprite\frameMax = 2
	sprite\frameOffset = 0
	sprite\frameCells = frameCells
	sprite\x = x
	sprite\y = y
	sprite\speedX = speedX#
	sprite\speedY = speedY#
	sprite\speedStep = speedStep#
	sprite\speedLimit = speedLimit
	sprite\speedBrake = speedBrake#
	
	sprite\oldSpeedBrake# = NO_SPEED#
	sprite\oldSpeedLimit# = NO_SPEED#
	
	sprite\width = width
	sprite\height = height
	sprite\frameCounterMax = frameCounterMax
	sprite\class = spriteClass
	sprite\animated = (frameCells > 0)
	sprite\imageName$ = imageName$
	sprite\oneDirectionalPusher = oneDirectionalPusher
	sprite\energy = 2
	sprite\master = master
	sprite\targetSpeedX = NO_SPEED#
	sprite\targetSpeedY = NO_SPEED#
	sprite\followTarget = False

	If spriteClass = CLASS_FOOL
		sprite\targetX = x
		sprite\targetY = y
	Else
		sprite\targetX = NO_POSITION
		sprite\targetY = NO_POSITION
	EndIf
	
	sprite\container = container
	sprite\containable = containable
	sprite\collideable = collideable
	sprite\contained = False
	sprite\layer = layer
	sprite\counter = COUNTER_END + 1
	
	sprite\classMorphEnergy = 0
	sprite\oldClass = NOBODY
	
	If sprite\animated
		sprite\image = getAnimImageFromCache("file\" + sprite\imageName$ + ".bmp", sprite\width, sprite\height, 0, frameCells)
	Else
		sprite\image = getImageFromCache("file\" + sprite\imageName$ + ".bmp")
	End If
	
	sprite\born = True
End Function 

Function handleSpeed(sprite.typeSprite)
	handleTargetSpeed sprite
	handleFollowTarget sprite
	sprite\oldX = sprite\x
	sprite\oldY = sprite\y
	sprite\x = sprite\x + sprite\speedX
	sprite\y = sprite\y + sprite\speedY
End Function

Function handleFollowTarget(sprite.typeSprite)
	If sprite\followTarget
		followTargetPosition sprite, sprite\targetX, sprite\targetY	
	EndIf
End Function

Function followTargetPosition(sprite.typeSprite, targetX, targetY)
	If targetX <> NO_POSITION
		If sprite\x < targetX
			sprite\speedX = sprite\speedX + sprite\speedStep#
		ElseIf sprite\x > targetX
			sprite\speedX = sprite\speedX - sprite\speedStep#
		EndIf
	EndIf
	
	If targetY <> NO_POSITION
		If sprite\y < targetY
			sprite\speedY = sprite\speedY + sprite\speedStep#
		ElseIf sprite\y > targetY
			sprite\speedY = sprite\speedY - sprite\speedStep#
		EndIf
	EndIf
End Function

Function handleTargetSpeed(sprite.typeSprite)
	If Not sprite\targetSpeedX = NO_SPEED
		targetStep# = .1
		If sprite\speedX < sprite\targetSpeedX
			sprite\speedX = sprite\speedX + targetStep
			If sprite\speedX >= sprite\targetSpeedX
				sprite\speedX = sprite\targetSpeedX
				sprite\targetSpeedX = NO_SPEED
			EndIf
		Else; if sprite\speedX# > sprite\targetSpeedX#
			sprite\speedX = sprite\speedX - targetStep
			If sprite\speedX <= sprite\targetSpeedX
				sprite\speedX = sprite\targetSpeedX
				sprite\targetSpeedX = NO_SPEED
			EndIf
		EndIf
	EndIf
	
	If Not sprite\targetSpeedY = NO_SPEED
		If sprite\speedY < sprite\targetSpeedY
			sprite\speedY = sprite\speedY + targetStep
			If sprite\speedY >= sprite\targetSpeedY
				sprite\speedY = sprite\targetSpeedY
				sprite\targetSpeedY = NO_SPEED
			EndIf
		Else; if sprite\speedX > sprite\targetSpeedX
			sprite\speedY = sprite\speedY - targetStep
			If sprite\speedY <= sprite\targetSpeedY
				sprite\speedY = sprite\targetSpeedY
				sprite\targetSpeedY = NO_SPEED
			EndIf
		EndIf
	EndIf
End Function

Function setFrameOffset(sprite.typeSprite)
	If sprite\frameCells = 8
		If sprite\speedX < 0
			offset = 0
		ElseIf sprite\speedX > 0
			offset = 1
		ElseIf sprite\speedY < 0
			offset = 2
		Else ;If sprite\speedY >= 0
			offset = 3
		EndIf
	Else ;If sprite\frameCells = 4
		If sprite\speedX < 0
			offset = 0
		Else
			offset = 1
		EndIf
	EndIf
	sprite\frameOffset = sprite\frame + offset * sprite\frameMax
End Function

Function isOutside(sprite.typeSprite)
	outsideX = (sprite\x + sprite\width < SCREEN_LEFT Or sprite\x > SCREEN_RIGHT)
	outsideY = (sprite\y + sprite\height < SCREEN_TOP Or sprite\y > SCREEN_BOTTOM)
	Return (outsideX Or outsideY)
End Function

Function spriteBorderBump(sprite.typeSprite)
	If sprite\x < SCREEN_LEFT
		sprite\speedX = sprite\speedX * -1
		sprite\x = SCREEN_LEFT
	ElseIf sprite\x + sprite\width > SCREEN_RIGHT-1
		sprite\speedX = sprite\speedX *-1
		sprite\x = SCREEN_RIGHT - sprite\width
	EndIf
	If sprite\y < SCREEN_TOP
		sprite\speedY = sprite\speedY * -1
		sprite\y = SCREEN_TOP
	ElseIf sprite\y + sprite\height > SCREEN_BOTTOM-1
		sprite\speedY = sprite\speedY * -1
		sprite\y = SCREEN_BOTTOM - sprite\height
	EndIf
End Function

Function spriteBorderRotate(sprite.typeSprite)
	If sprite\x < SCREEN_LEFT
		sprite\targetSpeedY = -Abs(sprite\speedLimit)
		sprite\speedX = 0
		sprite\x = SCREEN_LEFT
	ElseIf sprite\x + sprite\width > SCREEN_RIGHT - 1
		sprite\targetSpeedY = Abs(sprite\speedLimit)
		sprite\speedX = 0
		sprite\x = SCREEN_RIGHT - sprite\width - 1
	EndIf
	If sprite\y < SCREEN_TOP
		sprite\targetSpeedX = Abs(sprite\speedLimit)
		sprite\speedY = 0
		sprite\y = SCREEN_TOP
	ElseIf sprite\y + sprite\height > SCREEN_BOTTOM - 1
		sprite\targetSpeedX = -Abs(sprite\speedLimit)
		sprite\speedY = 0
		sprite\y = SCREEN_BOTTOM - sprite\height - 1
	EndIf
End Function

Function spriteBorderRestart(sprite.typeSprite)
	If sprite\x < SCREEN_LEFT - sprite\width
		sprite\x = SCREEN_RIGHT
	ElseIf sprite\x > SCREEN_RIGHT
		sprite\x = SCREEN_LEFT - sprite\width
	ElseIf sprite\y < SCREEN_TOP - sprite\height
		sprite\y = SCREEN_BOTTOM
	ElseIf sprite\y > SCREEN_BOTTOM
		sprite\y = SCREEN_TOP - sprite\height
	EndIf
End Function

Function spriteBorderBounce(sprite.typeSprite, soft)
	If soft Then
		border = TILE_SIZE
	Else
		border = 0
	EndIf
	
	If sprite\x < SCREEN_LEFT + border And sprite\speedX# < 0
		If sprite\targetSpeedX# = NO_SPEED
			sprite\targetSpeedX# = sprite\speedX# * -1
		EndIf
	ElseIf sprite\x + sprite\width > SCREEN_RIGHT - border And sprite\speedX# > 0
		If sprite\targetSpeedX# = NO_SPEED
			sprite\targetSpeedX# = sprite\speedX# * -1
		EndIf
	ElseIf sprite\y < SCREEN_TOP + border And sprite\speedY# < 0
		If sprite\targetSpeedY# = NO_SPEED
			sprite\targetSpeedY# = sprite\speedY# * -1
		EndIf
	ElseIf sprite\y + sprite\height > SCREEN_BOTTOM - border And sprite\speedY# < 0
		If sprite\targetSpeedY# = NO_SPEED
			sprite\targetSpeedY# = sprite\speedY# * -1
		EndIf
	EndIf
End Function

Function handleFrame(sprite.typeSprite)
	If sprite\animated 
		If Abs(sprite\speedX) > SPEED_MIN# Or Abs(sprite\speedY) > SPEED_MIN#
			sprite\frameCounter = sprite\frameCounter + 1
			If sprite\frameCounter > sprite\frameCounterMax
				sprite\frameCounter = 0
				sprite\frame = sprite\frame + 1
				If sprite\frame > sprite\frameMax-1
					sprite\frame = 0
				EndIf
			EndIf
		Else
			sprite\frame = 0
		EndIf
		setFrameOffset sprite
	EndIf
End Function

Function spriteInvertPosition(sprite.typeSprite)
	If sprite\x > SCREEN_WIDTH
		sprite\x = SCREEN_LEFT - sprite\width
	ElseIf sprite\x < SCREEN_LEFT - sprite\width
		sprite\x = SCREEN_WIDTH
	EndIf
	If sprite\y > SCREEN_HEIGHT
		sprite\y = SCREEN_TOP - sprite\height
	ElseIf sprite\y < SCREEN_TOP - sprite\height
		sprite\y = SCREEN_HEIGHT
	EndIf
End Function

Function pushPlayerInside()
	For sprite.typeSprite = Each typeSprite
		If sprite\class = CLASS_PLAYER
			If sprite\x + sprite\width < SCREEN_LEFT
				directionX = 1
			ElseIf sprite\x > SCREEN_RIGHT
				directionX = -1
			;Else
			;	directionX = 0
			EndIf
			If sprite\y + sprite\height < SCREEN_TOP
				directionY = 1
			ElseIf sprite\y > SCREEN_BOTTOM
				directionY = -1
			;Else
			;	directionY = 0
			EndIf

			If directionX <> 0 Or directionY <> 0
				sprite\pushedX# = directionX * sprite\speedLimit
				sprite\pushedY# = directionY * sprite\speedLimit
				sprite\pushedBy = CLASS_GOD
				morphTo sprite, CLASS_BALL, 30, 0.08, NO_SPEED#
			EndIf
			;Exit For
		EndIf
	Next
End Function

Function spriteClassExists(sprite.typeSprite, class)
	foundClass = False
	For sprite.typeSprite = Each typeSprite
		If sprite\class = class
			foundClass = True
		EndIf
	Next
	Return foundClass
End Function

Function drawSprites(sprite.typeSprite, shadeImage, shadeImageSmall)
	drawShadows sprite, shadeImage, shadeImageSmall
	For layer = LAYER_GROUND To LAYER_SKY
		For sprite.typeSprite = Each typeSprite
			If layer = sprite\layer Then
				If sprite\animated
					DrawImage sprite\image, sprite\x, sprite\y, sprite\frameOffset
				Else
					DrawImage sprite\image, sprite\x, sprite\y
				EndIf
			EndIf
		Next
	Next
End Function

Function drawShadows(sprite.typeSprite, shadeImage, shadeImageSmall)
	For sprite.typeSprite = Each typeSprite
		If (Not sprite\container) And sprite\collideable
			If sprite\width <= SPRITE_SMALL_SIZE Or sprite\height <= SPRITE_SMALL_SIZE
				If sprite\layer >= LAYER_NEAR_SKY
					yOffset = 10
				Else
					yOffset = 0
				EndIf
				DrawImage shadeImageSmall, sprite\x + SHADOW_SMALL_LEFT, sprite\y + SHADOW_SMALL_TOP + yOffset
			ElseIf Not sprite\class = CLASS_BARRIER
				DrawImage shadeImage, sprite\x + SHADOW_LEFT, sprite\y + SHADOW_TOP
			EndIf
		EndIf
	Next
End Function